import cv2
import time
import argparse
import numpy as np
import tritonclient.http as httpclient


def load_image(image_path, nw, nh):
    im = cv2.imread(image_path)
    im = cv2.cvtColor(im, cv2.COLOR_BGR2RGB)
    w = im.shape[0]
    h = im.shape[1]
    im = cv2.resize(im, (nw, nh))
    im = np.transpose(im, (2, 0, 1))
    im = np.expand_dims(im, axis=0)
    return np.ascontiguousarray(im, 'float32'), w, h


def recover_image(im, w, h):
    im = np.squeeze(im)
    im = np.transpose(im, (1, 2, 0))
    im = cv2.cvtColor(np.float32(im), cv2.COLOR_RGB2BGR)
    im = cv2.resize(im, (w, h))
    return im.astype(np.uint8)


def stylize(content_path, output_path, style='udnie'):
    triton_client = httpclient.InferenceServerClient(url='127.0.0.1:8000')
    style_to_version = {'udnie': '1', 'candy': '2', 'sketch': '3', 'mosaic': '4', 'rain_princess': '5'}
    image, w, h = load_image(content_path, 256, 256)
    print("input: ", w, h)
    inputs = []
    inputs.append(httpclient.InferInput('INPUT_0', image.shape, 'FP32'))
    inputs[0].set_data_from_numpy(image, binary_data=True)
    outputs = []
    outputs.append(httpclient.InferRequestedOutput('OUTPUT_0', binary_data=True))
    now = time.time()
    results = triton_client.infer('fast_neural_style', inputs=inputs,
                                  model_version=style_to_version[style], outputs=outputs)
    output0_data = results.as_numpy('OUTPUT_0')
    image = recover_image(output0_data, w, h)
    print("output: ", image.shape)
    cv2.imwrite(output_path, image)
    print((time.time() - now) * 1000, 'ms')


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--image',
                        required=True,
                        help='the image to classify')
    FLAGS = parser.parse_args()
    stylize(FLAGS.image, 'result.jpg')
