import os
import random
import json
import base64

from flask import Flask, render_template, request
from werkzeug.utils import secure_filename

from infer import stylize

app = Flask(__name__)
app.config["SECRET_KEY"] = "12345"
app.config["UPLOAD_FOLDER"] = "./temp"
base_dir = os.environ.get("BASE_DIR", "")
style_list = ["candy", "mosaic", "rain_princess", "sketch", "udnie"]


@app.route(f'{base_dir}/v1/debug', methods=['GET'])
def debug():
    with open('1.txt', 'r') as f:
        lines = f.readlines()
        return '<p>' + ''.join(lines) + '</p>'


@app.route(f"{base_dir}/v1/index", methods=["GET", "POST"])
def home():
    if request.method == "GET":
        return render_template("index.html", base_dir=base_dir)
    if request.method == "POST":
        style = request.form.get("style")
        if style not in style_list:
            json_obj = {"image_url": "", "prediction": "请输入正确风格类型"}
            return json.dumps(json_obj)
        if "image" not in request.files:
            json_obj = {"image_url": "", "prediction": "文件选择错误"}
            return json.dumps(json_obj)
        image_file = request.files["image"]

        if image_file.filename == "":
            json_obj = {"image_url": "", "prediction": "上传文件为空"}
            return json.dumps(json_obj)

        if image_file and is_allowed_file(image_file.filename):
            try:
                filename = generate_filenames(image_file.filename)
                filePath = os.path.join(app.config["UPLOAD_FOLDER"], filename)
                image_file.save(filePath)
                output_path = "." + filePath.split(".")[1] + "_" + style + ".jpg"
                # 推理
                stylize(filePath, output_path, style)
                with open(output_path, "rb") as img_f:
                    img_stream = img_f.read()
                    img_stream = base64.b64encode(img_stream).decode()
                return img_stream
            except Exception as e:
                json_obj = {"image_url": "", "prediction": "后台异常" + str(e)}
                return json.dumps(json_obj)


def is_allowed_file(filename):
    VALID_EXTENSIONS = ["jpg", "jpeg", "png"]
    is_valid_ext = filename.rsplit(".", 1)[1].lower() in VALID_EXTENSIONS
    return "." in filename and is_valid_ext


def generate_filenames(filename):
    LETTERS = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
    ext = filename.split(".")[-1]
    random_indexes = [random.randint(0, len(LETTERS) - 1) for _ in range(10)]
    random_chars = "".join([LETTERS[index] for index in random_indexes])
    new_name = "{name}.{extension}".format(name=random_chars, extension=ext)
    return secure_filename(new_name)


@app.errorhandler(500)
def server_error(error):
    return render_template("error.html"), 500


if __name__ == "__main__":
    print("server is running .....")
    app.run("0.0.0.0")
